/*
Licencia:
        Attribution-NonCommercial-ShareAlike 4.0 International (CC BY-NC-SA 4.0)
*/

package org.no_ip.arias.rockets;

import android.content.ContentValues;
import android.content.Context;
import android.database.Cursor;
import android.database.sqlite.SQLiteDatabase;
import android.database.sqlite.SQLiteOpenHelper;

import java.util.ArrayList;

public class AdministradorBD extends SQLiteOpenHelper {
    public static final int VERSION = 1;
    public static final String NOMBRE = "RocketApp.db";

    //Tabla Rockets
    public static final String TABLA_ROCKETS = "ROCKETS";
    public static final String ROCKET_ID = "rocket_id";
    public static final String ROCKET_NOMBRE = "nombre";
    public static final String ROCKET_COLOR = "color";
    public static final String ROCKET_APODO = "apodo";

    private static final String SQL_CREAR_TABLA_ROCKETS = "create table "+TABLA_ROCKETS+"("+ROCKET_ID+" integer primary key autoincrement, "+ROCKET_NOMBRE+" text not null, "+ROCKET_COLOR+" integer not null, "+ROCKET_APODO+" text not null);";

    //Tabla Recursos
    public static final String TABLA_RECURSOS = "RECURSOS";
    public static final String RECURSO_ID = "recurso_id";
    public static final String RECURSO_RUTA = "ruta";
    public static final String TAMANIO = "tamanio";
    public static final String TIPO = "tipo";
    public static final String NOMBRE_ARCHIVO = "nombre_archivo";
    public static final String TEXTO_CONTENIDO = "texto_contenido";

    private static final String SQL_CREAR_TABLA_RECURSOS = "create table "
            +TABLA_RECURSOS+"("+RECURSO_ID+" integer primary key autoincrement, "
            +ROCKET_ID+" integer not null, "
            +RECURSO_RUTA+" text, "
            +TAMANIO+" real, "
            +TIPO+" text, "
            +TEXTO_CONTENIDO+" text, "
            +NOMBRE_ARCHIVO+" text);";


    public AdministradorBD(Context context){
        super(context, NOMBRE, null, VERSION);
    }

    @Override
    public void onCreate(SQLiteDatabase sqLiteDatabase) {
        sqLiteDatabase.execSQL(SQL_CREAR_TABLA_ROCKETS);
        sqLiteDatabase.execSQL(SQL_CREAR_TABLA_RECURSOS);
    }

    @Override
    public void onUpgrade(SQLiteDatabase sqLiteDatabase, int i, int i1) {

    }

    public long insertarRocketDB(Rocket rocket){
        long id;

        SQLiteDatabase db = this.getWritableDatabase();

        ContentValues valores = new ContentValues();

        valores.put(ROCKET_NOMBRE, rocket.obtenerNombre());
        valores.put(ROCKET_COLOR, rocket.obtenerColor());
        valores.put(ROCKET_APODO, rocket.obtenerApodo());

        id = db.insert(TABLA_ROCKETS, null, valores);

        db.close();

        return id;
    }

    public Rocket obtenerRocketDB(long id){
        Rocket rocket = new Rocket();

        SQLiteDatabase db = this.getReadableDatabase();

        String proyeccion[] = {ROCKET_ID, ROCKET_NOMBRE, ROCKET_COLOR, ROCKET_APODO};

        Cursor cursor =
                db.query(TABLA_ROCKETS,
                        proyeccion,
                        " rocket_id = ?",
                        new String[] { String.valueOf(id)},
                        null,
                        null,
                        null,
                        null);

        if(cursor != null) {
            cursor.moveToFirst();
            rocket.establecerID(cursor.getInt(0));
            rocket.establecerNombre(cursor.getString(1));
            rocket.establecerColor(cursor.getInt(2));
            rocket.establecerApodo(cursor.getString(3));
        }
        cursor.close();
        db.close();
        return rocket;
    }

    public ArrayList<Rocket> obtenerRockets(){
        ArrayList<Rocket> listaRockets = new ArrayList<Rocket>();

        SQLiteDatabase db = this.getReadableDatabase();
        Cursor cursor = db.rawQuery("select * from ROCKETS", null);

        if(cursor.moveToFirst()){
            while(!cursor.isAfterLast()){
                Rocket rocket = new Rocket();
                rocket.establecerID(cursor.getInt(cursor.getColumnIndex(ROCKET_ID)));
                rocket.establecerNombre(cursor.getString(cursor.getColumnIndex(ROCKET_NOMBRE)));
                rocket.establecerColor(cursor.getInt(cursor.getColumnIndex(ROCKET_COLOR)));
                rocket.establecerApodo(cursor.getString(cursor.getColumnIndex(ROCKET_APODO)));
                listaRockets.add(rocket);
                cursor.moveToNext();
            }
        }

        cursor.close();
        db.close();
        return listaRockets;
    }

    public void actualizarRocketDB(long id, String nuevoApodo, int nuevoColor){
        SQLiteDatabase db = this.getWritableDatabase();

        ContentValues valores = new ContentValues();
        valores.put(ROCKET_APODO, nuevoApodo);
        valores.put(ROCKET_COLOR, nuevoColor);

        int i = db.update(TABLA_ROCKETS,
                valores,
                " rocket_id = ?",
                new String[] { String.valueOf(id) }
                );

        db.close();
    }

    public boolean elminarRocketDB(long id){
        SQLiteDatabase db = this.getWritableDatabase();

        try{
            //primero borrar todos los recursos del rocket
            db.delete(TABLA_RECURSOS, " rocket_id = ?",
                    new String[] { String.valueOf(id)});

            //finalmente borrar el rocket
            db.delete(TABLA_ROCKETS, " rocket_id = ?",
                    new String[] { String.valueOf(id)});

            db.close();
            return true;
        }catch (Exception ex){
            return false;
        }
    }

    public long contarRockets(){
        long n = 0;
        SQLiteDatabase db = this.getReadableDatabase();
        Cursor cursor = db.rawQuery("select count(*) from ROCKETS", null);
        cursor.moveToFirst();
        n = cursor.getInt(0);
        cursor.close();
        db.close();
        return n;
    }

    /******************
    Operaciones Recursos
    *******************/

    public ArrayList<Recurso> obtenerRecursos(long rocket_id){
        ArrayList<Recurso> listaRecursos = new ArrayList<Recurso>();

        SQLiteDatabase db = this.getReadableDatabase();
        Cursor cursor = db.rawQuery("select * from "+TABLA_RECURSOS+" where "+ROCKET_ID+"="+rocket_id, null);

        if(cursor.moveToFirst()){
            while(!cursor.isAfterLast()){
                Recurso recurso = new Recurso();
                recurso.establecerID(cursor.getInt(cursor.getColumnIndex(RECURSO_ID)));
                recurso.establecerRocket_ID(cursor.getInt(cursor.getColumnIndex(ROCKET_ID)));
                recurso.establecerNombreArchivo(cursor.getString(cursor.getColumnIndex(NOMBRE_ARCHIVO)));
                recurso.establecerRuta(cursor.getString(cursor.getColumnIndex(RECURSO_RUTA)));
                recurso.establecerTipo(cursor.getString(cursor.getColumnIndex(TIPO)));
                recurso.establecerTextoContenido(cursor.getString(cursor.getColumnIndex(TEXTO_CONTENIDO)));
                listaRecursos.add(recurso);
                cursor.moveToNext();
            }
        }

        cursor.close();
        db.close();
        return listaRecursos;
    }

    public long insertarRecursoDB(Recurso recurso, long rocket_id){
        long id;

        SQLiteDatabase db = this.getWritableDatabase();

        ContentValues valores = new ContentValues();

        valores.put(ROCKET_ID, rocket_id);
        valores.put(NOMBRE_ARCHIVO, recurso.obtenerNombre());
        valores.put(RECURSO_RUTA, recurso.obtenerRuta());
        valores.put(TIPO, recurso.obtenerTipo());
        valores.put(TAMANIO, recurso.obtenerTamanio());
        valores.put(TEXTO_CONTENIDO, recurso.obtenerTexto());

        id = db.insert(TABLA_RECURSOS, null, valores);

        db.close();

        return id;
    }

    public boolean elminarRecursoDB(long id){
        SQLiteDatabase db = this.getWritableDatabase();

        try{
            db.delete(TABLA_RECURSOS, " recurso_id = ?",
                    new String[] { String.valueOf(id)});
            db.close();
            return true;
        }catch (Exception ex){
            return false;
        }
    }



}
