/*
Licencia:
        Attribution-NonCommercial-ShareAlike 4.0 International (CC BY-NC-SA 4.0)
*/


package org.no_ip.arias.rockets;

import android.app.Activity;
import android.app.AlertDialog;
import android.content.ClipData;
import android.content.ClipboardManager;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.database.Cursor;
import android.graphics.Color;
import android.net.Uri;
import android.os.Bundle;
import android.os.Environment;
import android.os.StrictMode;
import android.provider.OpenableColumns;
import android.support.design.widget.FloatingActionButton;
import android.support.design.widget.Snackbar;
import android.support.v7.app.AppCompatActivity;
import android.support.v7.widget.Toolbar;
import android.util.Log;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.view.ViewGroup;
import android.webkit.MimeTypeMap;
import android.widget.ArrayAdapter;
import android.widget.EditText;
import android.widget.ImageButton;
import android.widget.ImageView;
import android.widget.ListView;
import android.widget.RadioButton;
import android.widget.TextView;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.List;
import java.util.zip.ZipEntry;
import java.util.zip.ZipOutputStream;

public class PantallaDetalle extends AppCompatActivity {
    static private int GMAIL_INTENT_REQUEST = 1;
    private static final int READ_REQUEST_CODE = 42;

    private String tag = "Actividad Detalle ";
    private long rocket_id;
    private String rocket_nombre, rocket_apodo;
    private Rocket rocket;
    private List<Recurso> listaRecursos;
    private ListView lista;
    private ImageView ivRecurso;
    private  Toolbar toolbar;
    private String nombreZIP;
    private ArrayList<String> listaTextos;
    private int contadorArchivos;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        StrictMode.VmPolicy.Builder builder = new StrictMode.VmPolicy.Builder();
        StrictMode.setVmPolicy(builder.build());

        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_pantalla_detalle);

        Intent intent = getIntent();
        rocket_id = intent.getLongExtra("rocket_id", 0);
        rocket_nombre = intent.getStringExtra("rocket_nombre");

        AdministradorBD db = new AdministradorBD(getApplicationContext());
        rocket = db.obtenerRocketDB(rocket_id);

        toolbar = (Toolbar) findViewById(R.id.toolbar);
        toolbar.setTitle("Recursos en "+rocket.obtenerApodo()+":");
        setSupportActionBar(toolbar);

        listaTextos = new ArrayList<String>();
        contadorArchivos = 0;

        lista = (ListView)findViewById(R.id.listaRecursos);

        cargarRecursos();

        FloatingActionButton fab = (FloatingActionButton) findViewById(R.id.fab);
        fab.setImageResource(R.drawable.plus_icon);

        fab.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                //agregar archivo usando el selector de Android
                Intent intent = new Intent(Intent.ACTION_GET_CONTENT);
                intent.addCategory(Intent.CATEGORY_OPENABLE);
                intent.setType("*/*");
                startActivityForResult(intent, READ_REQUEST_CODE);
            }
        });
    }

    private void enviarEmail(File archivo){
        Intent intentEnviar = new Intent(Intent.ACTION_SEND);
        intentEnviar.setType("message/rfc822");
        //intentEnviar.setClassName("com.google.android.gm", "com.google.android.gm.ComposeActivityGmail");
        intentEnviar.putExtra(Intent.EXTRA_SUBJECT, "Rocket "+rocket.obtenerApodo());
        Uri uriZip = Uri.fromFile(archivo);
        intentEnviar.putExtra(Intent.EXTRA_STREAM, uriZip);

        //agregar los recursos de texto si es que los hay
        for(int i=0; i<listaTextos.size(); i++){
            intentEnviar.putExtra(Intent.EXTRA_TEXT, listaTextos.get(i));
        }
        //intentEnviar.setType("application/zip");
        //startActivity(intentEnviar);
        startActivityForResult(intentEnviar, GMAIL_INTENT_REQUEST);
    }

    private void enviarEmailSoloTexto(){
        Intent intentEnviar = new Intent(Intent.ACTION_SEND);
        intentEnviar.setType("message/rfc822");
        intentEnviar.putExtra(Intent.EXTRA_SUBJECT, "Rocket "+rocket.obtenerApodo());

        //agregar los recursos de texto si es que los hay
        String cuerpo = "";
        for(int i=0; i<listaTextos.size(); i++){
            cuerpo = cuerpo + listaTextos.get(i) + "\n\n";
        }
        intentEnviar.putExtra(Intent.EXTRA_TEXT, cuerpo);
        startActivityForResult(intentEnviar, GMAIL_INTENT_REQUEST);
    }

    private void cargarRecursos(){
        AdministradorBD administradorBD = new AdministradorBD(getApplicationContext());

        listaRecursos = administradorBD.obtenerRecursos(rocket_id);
        Log.d(tag, "N: "+listaRecursos.size());

        ArrayAdapter<Recurso> adaptador = new ArrayAdapter<Recurso>(this, R.layout.esquema_filas_recursos, R.id.textoRecurso, listaRecursos){
            @Override
            public View getView(int position, View convertView, ViewGroup parent){
                View view = super.getView(position, convertView, parent);
                final Recurso recurso = getItem(position);

                ivRecurso = (ImageView)view.findViewById(R.id.ivRecurso);
                if(recurso.obtenerTipo().equals("IMG")){
                    ivRecurso.setImageDrawable(getResources().getDrawable(R.mipmap.picture, PantallaDetalle.this.getTheme()));
                    contadorArchivos++;
                }else if(recurso.obtenerTipo().equals("PDF")){
                    ivRecurso.setImageDrawable(getResources().getDrawable(R.mipmap.pdf, PantallaDetalle.this.getTheme()));
                    contadorArchivos++;
                }else if(recurso.obtenerTipo().equals("TXT")){
                    ivRecurso.setImageDrawable(getResources().getDrawable(R.mipmap.texto_icon, PantallaDetalle.this.getTheme()));
                    listaTextos.add(recurso.obtenerTexto());
                }else if(recurso.obtenerTipo().equals("AUD")){
                    ivRecurso.setImageDrawable(getResources().getDrawable(R.mipmap.audio_icon, PantallaDetalle.this.getTheme()));
                    contadorArchivos++;
                }else if(recurso.obtenerTipo().equals("VID")){
                    ivRecurso.setImageDrawable(getResources().getDrawable(R.mipmap.video_icon, PantallaDetalle.this.getTheme()));
                    contadorArchivos++;
                }else if(recurso.obtenerTipo().equals("DOC")){
                    ivRecurso.setImageDrawable(getResources().getDrawable(R.mipmap.doc_icon, PantallaDetalle.this.getTheme()));
                    contadorArchivos++;
                }else if(recurso.obtenerTipo().equals("LNK")){
                    ivRecurso.setImageDrawable(getResources().getDrawable(R.mipmap.link_icon, PantallaDetalle.this.getTheme()));
                    listaTextos.add(recurso.obtenerTexto());
                }else{
                    ivRecurso.setImageDrawable(getResources().getDrawable(R.mipmap.otro_icon, PantallaDetalle.this.getTheme()));
                    contadorArchivos++;
                }

                TextView tv = (TextView)view.findViewById(R.id.textoRecurso);
                tv.setText(recurso.obtenerNombre());



                tv.setOnClickListener(new View.OnClickListener() {
                    @Override
                    public void onClick(View view) {
                        //Abrir siguiente activity y pasar ID
                        //Log.d(tag, "Recurso: "+recurso.obtenerID());
                        //Log.d(tag, "Ruta DB: "+recurso.obtenerRuta());
                        if(recurso.obtenerTipo().equals("TXT") || recurso.obtenerTipo().equals("LNK")){
                            Log.d(tag, "Texto: "+recurso.obtenerTexto());
                            String whatsNewTitle = "Texto Guardado";
                            String whatsNewText = recurso.obtenerTexto();
                            new AlertDialog.Builder(PantallaDetalle.this).setIcon(R.mipmap.rocket_icon).setTitle(whatsNewTitle).setMessage(whatsNewText).setPositiveButton(
                                    "Aceptar", new DialogInterface.OnClickListener() {
                                        public void onClick(DialogInterface dialog, int which) {
                                            dialog.dismiss();
                                        }
                                    }).setNegativeButton("Copiar", new DialogInterface.OnClickListener() {
                                public void onClick(DialogInterface dialog, int which) {
                                    ClipboardManager clipboardManager = (ClipboardManager) getSystemService(Context.CLIPBOARD_SERVICE);
                                    ClipData clipData = ClipData.newPlainText("Texto Plano", recurso.obtenerTexto());
                                    clipboardManager.setPrimaryClip(clipData);
                                }
                            }).show();
                        }else{
                            //Crear URI
                            String rutaArchivo = getExternalFilesDir(Environment.DIRECTORY_DOCUMENTS) + "/" + rocket_id + "_" + rocket_nombre + "/" + recurso.obtenerNombre();
                            Log.d(tag, "Ruta:" + rutaArchivo);
                            Uri uriImagen = Uri.fromFile(new File(rutaArchivo));

                            //Crear Intent para Abrir
                            if (recurso.obtenerTipo().equals("IMG")) {
                                Intent i = new Intent();
                                i.setAction(Intent.ACTION_VIEW);
                                i.setDataAndType(Uri.fromFile(new File(rutaArchivo)), "image/*");
                                startActivity(i);
                            } else if (recurso.obtenerTipo().equals("PDF")) {
                                Intent i = new Intent();
                                i.setAction(Intent.ACTION_VIEW);
                                i.setDataAndType(Uri.fromFile(new File(rutaArchivo)), "application/pdf");
                                startActivity(i);
                            } else if(recurso.obtenerTipo().equals("VID")){
                                Intent i = new Intent();
                                i.setAction(Intent.ACTION_VIEW);
                                i.setDataAndType(Uri.fromFile(new File(rutaArchivo)), "video/*");
                                startActivity(i);
                            } else if(recurso.obtenerTipo().equals("AUD")){
                                Intent i = new Intent();
                                i.setAction(Intent.ACTION_VIEW);
                                i.setDataAndType(Uri.fromFile(new File(rutaArchivo)), "audio/*");
                                startActivity(i);
                            } else if(recurso.obtenerTipo().equals("DOC")){
                                Intent i = new Intent();
                                i.setAction(Intent.ACTION_VIEW);
                                i.setDataAndType(Uri.fromFile(new File(rutaArchivo)), "application/*");
                                startActivity(i);
                            }
                        }
                   }
                });

                ImageButton btnEliminar = (ImageButton)view.findViewById(R.id.btnEliminarRecurso);

                btnEliminar.setOnClickListener(new View.OnClickListener() {
                    @Override
                    public void onClick(View view) {
                        //dialogo para confirmar eliminar
                        mostrarDialogoEliminarRecurso(recurso, "Está seguro que desea eliminar este Recurso? Esta acción no se puede deshacer.");
                    }
                });

                return view;
            }
        };

        lista.setAdapter(adaptador);
    }

    /* Funciones de compresión obtenidas de: https://stackoverflow.com/questions/6683600/zip-compress-a-folder-full-of-files-on-android/14868161#14868161
        Autor: https://stackoverflow.com/users/2053024/hailzeon
     */

    public boolean zipFileAtPath(String sourcePath, String toLocation) {
        final int BUFFER = 2048;

        File sourceFile = new File(sourcePath);
        try {
            BufferedInputStream origin = null;
            FileOutputStream dest = new FileOutputStream(toLocation);
            ZipOutputStream out = new ZipOutputStream(new BufferedOutputStream(
                    dest));
            if (sourceFile.isDirectory()) {
                zipSubFolder(out, sourceFile, sourceFile.getParent().length());
            } else {
                byte data[] = new byte[BUFFER];
                FileInputStream fi = new FileInputStream(sourcePath);
                origin = new BufferedInputStream(fi, BUFFER);
                ZipEntry entry = new ZipEntry(getLastPathComponent(sourcePath));
                entry.setTime(sourceFile.lastModified()); // to keep modification time after unzipping
                out.putNextEntry(entry);
                int count;
                while ((count = origin.read(data, 0, BUFFER)) != -1) {
                    out.write(data, 0, count);
                }
            }
            out.close();
        } catch (Exception e) {
            e.printStackTrace();
            Log.d(tag, "Drama:"+e.toString());
            return false;
        }
        return true;
    }

    /*
     *
     * Zips a subfolder
     *
     */

    private void zipSubFolder(ZipOutputStream out, File folder,
                              int basePathLength) throws IOException {

        final int BUFFER = 2048;

        File[] fileList = folder.listFiles();
        BufferedInputStream origin = null;
        for (File file : fileList) {
            if (file.isDirectory()) {
                zipSubFolder(out, file, basePathLength);
            } else {
                byte data[] = new byte[BUFFER];
                String unmodifiedFilePath = file.getPath();
                String relativePath = unmodifiedFilePath
                        .substring(basePathLength);
                FileInputStream fi = new FileInputStream(unmodifiedFilePath);
                origin = new BufferedInputStream(fi, BUFFER);
                ZipEntry entry = new ZipEntry(relativePath);
                entry.setTime(file.lastModified()); // to keep modification time after unzipping
                out.putNextEntry(entry);
                int count;
                while ((count = origin.read(data, 0, BUFFER)) != -1) {
                    out.write(data, 0, count);
                }
                origin.close();
            }
        }
    }

    /*
     * gets the last path component
     *
     * Example: getLastPathComponent("downloads/example/fileToZip");
     * Result: "fileToZip"
     */
    public String getLastPathComponent(String filePath) {
        String[] segments = filePath.split("/");
        if (segments.length == 0)
            return "";
        String lastPathComponent = segments[segments.length - 1];
        return lastPathComponent;
    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        // Inflate the menu; this adds items to the action bar if it is present.
        getMenuInflater().inflate(R.menu.menu_pantalla_detalle, menu);
        return true;
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        // Handle action bar item clicks here. The action bar will
        // automatically handle clicks on the Home/Up button, so long
        // as you specify a parent activity in AndroidManifest.xml.
        int id = item.getItemId();

        //noinspection SimplifiableIfStatement
        if (id == R.id.accion_modificar) {
            mostrarDialogoModificarRocket(getCurrentFocus());
            return true;
        }else if(id == R.id.accion_enviar){
            if(contadorArchivos > 0) {
                //enviar por email
                String rutaEntrada = getExternalFilesDir(Environment.DIRECTORY_DOCUMENTS).toString() + "/" + rocket_id + "_" + rocket_nombre;
                String archivoSalida = getExternalFilesDir(Environment.DIRECTORY_DOCUMENTS).toString() + "/" + rocket_id + "_" + rocket.obtenerApodo() + ".zip";
                nombreZIP = archivoSalida; //copia global para eliminar luego
                boolean comprimido = zipFileAtPath(rutaEntrada, archivoSalida);
                if (comprimido) {
                    File archivoZip = new File(archivoSalida);
                    long pesoArchivo = archivoZip.length() / (1024 * 1024); //MB
                    Log.d(tag, "Archivo:" + archivoSalida);
                    Log.d(tag, "Peso:" + archivoZip.length());
                    if (pesoArchivo < 20) {
                        Log.d(tag, "Listo para enviar");
                        enviarEmail(archivoZip);
                    } else {
                        Log.d(tag, "Archivo excede el adjunto max.");
                    }
                } else {
                    Log.d(tag, "No comprimido.");
                }
            }else{
                enviarEmailSoloTexto();
            }
        }

        return super.onOptionsItemSelected(item);
    }

    private void mostrarDialogoModificarRocket(final View view){

        AlertDialog.Builder builder = new AlertDialog.Builder(this);
        builder.setTitle("Modificar Rocket");

        final View customLayout = getLayoutInflater().inflate(R.layout.dialogo_nuevo_rocket, null);
        EditText nombre_et = (EditText)customLayout.findViewById(R.id.nombre_dialogo);
        nombre_et.setText(rocket.obtenerApodo(), TextView.BufferType.EDITABLE);
        builder.setView(customLayout);

        builder.setPositiveButton("Modificar", new DialogInterface.OnClickListener() {
            @Override
            public void onClick(DialogInterface dialogInterface, int i) {
                EditText nombre_et = (EditText)customLayout.findViewById(R.id.nombre_dialogo);
                RadioButton radioBtn1 = (RadioButton)customLayout.findViewById(R.id.radioBtn1);
                RadioButton radioBtn2 = (RadioButton)customLayout.findViewById(R.id.radioBtn2);
                RadioButton radioBtn3 = (RadioButton)customLayout.findViewById(R.id.radioBtn3);
                RadioButton radioBtn4 = (RadioButton)customLayout.findViewById(R.id.radioBtn4);

                int color=1;
                if(radioBtn1.isChecked())
                    color = 1;
                else if(radioBtn2.isChecked())
                    color = 2;
                else if(radioBtn3.isChecked())
                    color = 3;
                else if(radioBtn4.isChecked())
                    color = 4;



                AdministradorBD db = new AdministradorBD(getApplicationContext());
                db.actualizarRocketDB(rocket_id, nombre_et.getText().toString(), color);
                Log.d(tag, "Rocket:"+rocket_id+" actualizado.");
                toolbar.setTitle("Recursos en "+nombre_et.getText().toString()+":");
            }
        });

        builder.setNegativeButton("Cancelar", new DialogInterface.OnClickListener() {
            @Override
            public void onClick(DialogInterface dialogInterface, int i) {
                //AdministradorBD db = new AdministradorBD(getApplicationContext());
                //Log.d(tag, "Total Rockets: "+db.contarRockets());
                //mostrarSnack(view, "Rocket no creado.");
            }
        });

        builder.create();

        builder.show();
    }

    private void mostrarSnack(View view, String msg){
        Snackbar.make(view, msg, Snackbar.LENGTH_LONG)
                .setAction("Action", null).show();
    }

    private void mostrarDialogoEliminarRecurso(final Recurso recurso, String mensaje){
        AlertDialog.Builder dialog = new AlertDialog.Builder(this);
        dialog.setTitle( "Eliminar Recurso" )
                .setMessage(mensaje)
                .setNegativeButton("Cancelar", new DialogInterface.OnClickListener() {
                    public void onClick(DialogInterface dialoginterface, int i) {
                        dialoginterface.cancel();
                    }})
                .setPositiveButton("Ok", new DialogInterface.OnClickListener() {
                    public void onClick(DialogInterface dialoginterface, int i) {
                        eliminarRecurso(recurso);
                    }
                }).show();
    }

    private void eliminarRecurso(Recurso recurso){
        Log.d(tag, "Eliminar"+recurso.obtenerID());
        AdministradorBD db = new AdministradorBD(getApplicationContext());
        db.elminarRecursoDB(recurso.obtenerID());
        cargarRecursos();

        //eliminar archivo
        String rutaImagen = getExternalFilesDir(Environment.DIRECTORY_DOCUMENTS)+"/"+rocket_id+"_"+rocket_nombre+"/"+recurso.obtenerNombre();
        File archivo = new File(rutaImagen);
        boolean borrado = archivo.delete();
        if(borrado){
            Log.d(tag, "Borrado");
        }else{
            Log.d(tag, "No borrado");
        }

    }

    @Override
    protected void onActivityResult(int requestCode, int resultCode, Intent data) {
        if (requestCode == GMAIL_INTENT_REQUEST) {
            Log.d(tag, "E-mail enviado!");
            //eliminar archivo zip
            //File archivo = new File(nombreZIP);
            boolean borrado = false; //archivo.delete();
            if(borrado){
                Log.d(tag, "ZIP borrado.");
            }else{
                Log.d(tag, "ZIP no borrado.");
            }
        }else if(requestCode == READ_REQUEST_CODE && resultCode == Activity.RESULT_OK){
            Uri uri = null;
            if(data != null){
                uri = data.getData();
                Recurso recurso = new Recurso();
                recurso.establecerNombreArchivo(obtenerNombreArchivo(uri));
                recurso.establecerRocket_ID(rocket.obtenerID());
                recurso.establecerRuta(this.getExternalFilesDir(Environment.DIRECTORY_DOCUMENTS)+"/"+rocket.obtenerID()+"_"+rocket.obtenerNombre());
                recurso.establecerTipo(obtenerTipo(obtenerNombreArchivo(uri)));

                File archivoImagen = new File(this.getExternalFilesDir(Environment.DIRECTORY_DOCUMENTS)+"/"+rocket.obtenerID()+"_"+rocket.obtenerNombre(), obtenerNombreArchivo(uri));
                Log.d(tag, "Archivo: "+archivoImagen.toString());
                try {
                    InputStream entrada = getContentResolver().openInputStream(uri);
                    OutputStream salida = new FileOutputStream(archivoImagen);
                    byte[] buf = new byte[1024];
                    int len, tamanio = 0;
                    while((len=entrada.read(buf))>0){
                        salida.write(buf,0,len);
                        tamanio+=1024;
                    }
                    Log.d(tag, "Porte: "+tamanio);
                    recurso.establecerTamanio(tamanio);
                    salida.close();
                    entrada.close();

                    AdministradorBD db = new AdministradorBD(getApplicationContext());
                    long id = db.insertarRecursoDB(recurso, rocket.obtenerID());
                    cargarRecursos();
                }catch (Exception ex){
                    Log.d(tag, "Error manipulando archivo.");
                    Log.d(tag, ex.toString());
                }
            }
        }
    }

    private String obtenerNombreArchivo(Uri uri) {
        String result = null;
        if (uri.getScheme().equals("content")) {
            Cursor cursor = getContentResolver().query(uri, null, null, null, null);
            try {
                if (cursor != null && cursor.moveToFirst()) {
                    result = cursor.getString(cursor.getColumnIndex(OpenableColumns.DISPLAY_NAME));
                }
            } finally {
                cursor.close();
            }
        }
        if (result == null) {
            result = uri.getPath();
            int cut = result.lastIndexOf('/');
            if (cut != -1) {
                result = result.substring(cut + 1);
            }
        }
        return result;
    }

    private String obtenerTipo(String nombreArchivo){
        String extensionArchivo = nombreArchivo.substring(nombreArchivo.length()-3).toLowerCase();
        if(extensionArchivo.equals("pdf"))
            return "PDF";
        else if(extensionArchivo.equals("jpg") || extensionArchivo.equals("png") || extensionArchivo.equals("gif") || extensionArchivo.equals("bmp")) {
            return "IMG";
        }else if(extensionArchivo.equals("mp4") || extensionArchivo.equals("avi") || extensionArchivo.equals("mpg") || extensionArchivo.equals("mkv")){
            return "VID";
        }else if(extensionArchivo.equals("mp3") || extensionArchivo.equals("ogg") || extensionArchivo.equals("wav") || extensionArchivo.equals("acc")){
            return "AUD";
        }else if(extensionArchivo.equals("doc") || extensionArchivo.equals("xls") || extensionArchivo.equals("odt") || extensionArchivo.equals("ods") || extensionArchivo.equals("ppt")){
            return "DOC";
        }else{
            return "OTRO";
        }
    }
}
