/*
Licencia:
        Attribution-NonCommercial-ShareAlike 4.0 International (CC BY-NC-SA 4.0)
*/


package org.no_ip.arias.rockets;

import android.Manifest;
import android.app.Dialog;
import android.arch.lifecycle.Lifecycle;
import android.arch.lifecycle.OnLifecycleEvent;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.SharedPreferences;
import android.content.pm.PackageManager;
import android.graphics.Color;
import android.os.Build;
import android.os.Bundle;
import android.os.Debug;
import android.os.Environment;
import android.preference.PreferenceManager;
import android.support.design.widget.FloatingActionButton;
import android.support.design.widget.Snackbar;
import android.support.v4.app.ActivityCompat;
import android.support.v4.content.ContextCompat;
import android.support.v7.app.AppCompatActivity;
import android.support.v7.widget.Toolbar;
import android.util.Log;
import android.view.View;
import android.view.Menu;
import android.view.MenuItem;
import android.app.AlertDialog;
import android.view.LayoutInflater;
import android.view.ViewGroup;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.EditText;
import android.widget.ImageButton;
import android.widget.ImageView;
import android.widget.ListView;
import android.widget.RadioButton;
import android.widget.TextView;
import android.widget.Toast;

import org.w3c.dom.Text;

import java.io.File;
import java.util.ArrayList;
import java.util.List;

public class PantallaPrincipal extends AppCompatActivity {
    private String tag = "Actividad Principal ";
    private EditText nombre_et;
    private RadioButton radioBtn1, radioBtn2, radioBtn3, radioBtn4;
    private ImageView imageView1, imageView2, imageView3;

    private TextView consejoTv;
    private ListView lista;
    private List<Rocket> listaRockets;
    private static final int PERMISSION_REQUEST_CODE = 1;
    private SharedPreferences preferencias;
    final String prefBienvenidaMostrada = "bienvenidaMostrada";

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_pantalla_principal);

        if (Build.VERSION.SDK_INT >= 23)
        {
            if (checkPermission())
            {
                // Code for above or equal 23 API Oriented Device
                // Your Permission granted already .Do next code
            } else {
                requestPermission(); // Code for permission
            }
        }

        //Pantalla de Bienvenida
        preferencias = PreferenceManager.getDefaultSharedPreferences(this);
        Boolean bienvenidaMostrada = preferencias.getBoolean(prefBienvenidaMostrada, false);
        consejoTv = (TextView) findViewById(R.id.textoInicial);

        if(!bienvenidaMostrada){
            //mostrar pantalla de bienvenida
            String whatsNewTitle = "Bienvenido";
            String whatsNewText = "Rockets es una aplicación que crea copias de tus archivos originales para que los ordenes por temáticas. Cuando eliminas un archivo de un Rocket, el archivo original lo sigues manteniendo ;-)";
            new AlertDialog.Builder(this).setIcon(R.mipmap.rocket_icon).setTitle(whatsNewTitle).setMessage(whatsNewText).setPositiveButton(
                    "Entendido", new DialogInterface.OnClickListener() {
                        public void onClick(DialogInterface dialog, int which) {
                            dialog.dismiss();
                        }
                    }).show();

            SharedPreferences.Editor editor = preferencias.edit();
            editor.putBoolean(prefBienvenidaMostrada, true);
            editor.commit();

            consejoTv.setText("Crea tu primer Rocket presionando el botón +");
        }else{
            consejoTv.setText("");
        }


        Toolbar toolbar = (Toolbar) findViewById(R.id.toolbar);
        setSupportActionBar(toolbar);

        FloatingActionButton fab = (FloatingActionButton) findViewById(R.id.fab);
        fab.setImageResource(R.drawable.plus_icon);

        //total_tv = (TextView)findViewById(R.id.total_tv);
        lista = (ListView)findViewById(R.id.lista);

        cargarRockets();

        fab.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                mostrarDialogoCrearRocket(view);
            }
        });

        lista.setOnItemClickListener( new android.widget.AdapterView.OnItemClickListener() {
            @Override
            public void onItemClick(AdapterView<?> parent, View view, int position, long id){
                Rocket item = (Rocket)lista.getItemAtPosition(position);
                Log.d(tag, "Rocket: "+item.obtenerID());
            }
        });


    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        // Inflate the menu; this adds items to the action bar if it is present.
        //getMenuInflater().inflate(R.menu.menu_pantalla_principal, menu);
        return true;
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        // Handle action bar item clicks here. The action bar will
        // automatically handle clicks on the Home/Up button, so long
        // as you specify a parent activity in AndroidManifest.xml.
        int id = item.getItemId();

        //noinspection SimplifiableIfStatement
        if (id == R.id.action_settings) {
            return true;
        }

        return super.onOptionsItemSelected(item);
    }

    private void cargarRockets(){
        AdministradorBD administradorBD = new AdministradorBD(getApplicationContext());

        listaRockets = administradorBD.obtenerRockets();

        ArrayAdapter<Rocket> adaptador = new ArrayAdapter<Rocket>(this, R.layout.esquema_fila, R.id.textoLista, listaRockets){
            @Override
            public View getView(int position, View convertView, ViewGroup parent){
                View view = super.getView(position, convertView, parent);
                final Rocket rocket = getItem(position);

                TextView tv = (TextView)view.findViewById(R.id.textoLista);
                imageView1 = (ImageView)view.findViewById(R.id.iv1);
                imageView2 = (ImageView)view.findViewById(R.id.iv2);
                imageView3 = (ImageView)view.findViewById(R.id.iv3);
                asignarIcono(imageView1, rocket, 0);
                asignarIcono(imageView2, rocket, 1);
                asignarIcono(imageView3, rocket, 2);


                tv.setText(rocket.obtenerApodo());

                tv.setOnClickListener(new View.OnClickListener() {
                    @Override
                    public void onClick(View view) {
                        //Abrir siguiente activity y pasar ID
                        Log.d(tag, "Rocket: "+rocket.obtenerID());
                        Intent intent = new Intent(PantallaPrincipal.this, PantallaDetalle.class);
                        intent.putExtra("rocket_id", rocket.obtenerID());
                        intent.putExtra("rocket_nombre", rocket.obtenerNombre());
                        startActivity(intent);
                        //finish();
                    }
                });

                ImageButton btnEliminar = (ImageButton)view.findViewById(R.id.btnEliminar);

                btnEliminar.setOnClickListener(new View.OnClickListener() {
                    @Override
                    public void onClick(View view) {
                        //dialogo para confirmar eliminar
                        mostrarDialogoEliminarRocket(rocket, "Está seguro que desea eliminar este Rocket y todo su contenido? Esta acción no se puede deshacer.");



                    }
                });

                if(rocket.obtenerColor() == 1)
                    view.setBackgroundColor(getResources().getColor(R.color.color1, PantallaPrincipal.this.getTheme()));
                else if(rocket.obtenerColor() == 2)
                    view.setBackgroundColor(getResources().getColor(R.color.color2, PantallaPrincipal.this.getTheme()));
                else if(rocket.obtenerColor() == 3)
                    view.setBackgroundColor(getResources().getColor(R.color.color3, PantallaPrincipal.this.getTheme()));
                else
                    view.setBackgroundColor(getResources().getColor(R.color.color4, PantallaPrincipal.this.getTheme()));

                return view;
            }
        };

        lista.setAdapter(adaptador);
    }

    private void mostrarDialogoCrearRocket(final View view){

        AlertDialog.Builder builder = new AlertDialog.Builder(this);
        builder.setTitle("Crear Rocket");

        final View customLayout = getLayoutInflater().inflate(R.layout.dialogo_nuevo_rocket, null);

        builder.setView(customLayout);

        builder.setPositiveButton("Crear", new DialogInterface.OnClickListener() {
            @Override
            public void onClick(DialogInterface dialogInterface, int i) {
                nombre_et = (EditText)customLayout.findViewById(R.id.nombre_dialogo);
                radioBtn1 = (RadioButton)customLayout.findViewById(R.id.radioBtn1);
                radioBtn2 = (RadioButton)customLayout.findViewById(R.id.radioBtn2);
                radioBtn3 = (RadioButton)customLayout.findViewById(R.id.radioBtn3);
                radioBtn4 = (RadioButton)customLayout.findViewById(R.id.radioBtn4);

                int color=1;
                if(radioBtn1.isChecked())
                    color = 1;
                else if(radioBtn2.isChecked())
                    color = 2;
                else if(radioBtn3.isChecked())
                    color = 3;
                else if(radioBtn4.isChecked())
                    color = 4;

                String nombreRocket = nombre_et.getText().toString();
                Rocket rocket = new Rocket();
                if(nombreRocket.length() == 0) {
                    rocket.establecerNombre("Rocket");
                    rocket.establecerApodo("Rocket");
                }else{
                    rocket.establecerNombre(nombreRocket);
                    rocket.establecerApodo(nombreRocket);
                }
                rocket.establecerColor(color);

                AdministradorBD db = new AdministradorBD(getApplicationContext());
                long id = db.insertarRocketDB(rocket);
                Log.d(tag, "ID:"+id);

                rocket = db.obtenerRocketDB(id);
                Log.d(tag, rocket.obtenerNombre()+ " " +rocket.obtenerColor());

                crearDirectorio(rocket);

                cargarRockets();

                mostrarSnack(view, "Rocket creado.");
                consejoTv.setText("");
            }
        });

        builder.setNegativeButton("Cancelar", new DialogInterface.OnClickListener() {
            @Override
            public void onClick(DialogInterface dialogInterface, int i) {
                //AdministradorBD db = new AdministradorBD(getApplicationContext());
                //Log.d(tag, "Total Rockets: "+db.contarRockets());
                mostrarSnack(view, "Rocket no creado.");
            }
        });

        builder.create();

        builder.show();
    }

    private void mostrarDialogoEliminarRocket(final Rocket rocket, String mensaje){
        AlertDialog.Builder dialog = new AlertDialog.Builder(this);
        dialog.setTitle( "Eliminar Rocket" )
                .setMessage(mensaje)
                 .setNegativeButton("Cancelar", new DialogInterface.OnClickListener() {
                public void onClick(DialogInterface dialoginterface, int i) {
                    dialoginterface.cancel();
                }})
                .setPositiveButton("Ok", new DialogInterface.OnClickListener() {
                    public void onClick(DialogInterface dialoginterface, int i) {
                        eliminarRocket(rocket);
                    }
                }).show();
    }

    private void eliminarRocket(Rocket rocket){

        Log.d(tag, "Eliminar"+rocket.obtenerID());

        //borrar primero todos los archivos fisicos de la memoria
        AdministradorBD db = new AdministradorBD(getApplicationContext());
        ArrayList<Recurso> listaRecursos = db.obtenerRecursos(rocket.obtenerID());
        for(int i=0; i<listaRecursos.size(); i++){
            String rutaImagen = listaRecursos.get(i).obtenerRuta()+"/"+listaRecursos.get(i).obtenerNombre();
            File archivo = new File(rutaImagen);
            boolean borrado = archivo.delete();
            if(borrado){
                Log.d(tag, "Borrado");
            }else{
                Log.d(tag, "No borrado");
            }
        }

        //borrar el directorio
        String rutaDirectorio = getExternalFilesDir(Environment.DIRECTORY_DOCUMENTS).toString()+"/"+rocket.obtenerID()+"_"+rocket.obtenerNombre();
        File directorio = new File(rutaDirectorio);
        boolean borrado = directorio.delete();
        Log.d(tag, "Directorio: "+rutaDirectorio);
        Log.d(tag, "Directorio borrado: "+borrado);

        //borrar el rocket y sus recursos de la BD
        db.elminarRocketDB(rocket.obtenerID());
        cargarRockets();
    }

    private void crearDirectorio(Rocket rocket){
        File file = new File(this.getExternalFilesDir(Environment.DIRECTORY_DOCUMENTS), rocket.obtenerID()+"_"+rocket.obtenerNombre());
        if (!file.mkdir()) {
            Log.d(tag, "Operacion fallida.");
        }else{
            Log.d(tag, "Operacion Correcta.");
            Log.d(tag, "Ruta:"+file.getPath());
        }
    }

    private boolean checkPermission() {
        int result = ContextCompat.checkSelfPermission(PantallaPrincipal.this, android.Manifest.permission.WRITE_EXTERNAL_STORAGE);
        if (result == PackageManager.PERMISSION_GRANTED) {
            return true;
        } else {
            return false;
        }
    }

    private void requestPermission() {
        if (ActivityCompat.shouldShowRequestPermissionRationale(PantallaPrincipal.this, android.Manifest.permission.WRITE_EXTERNAL_STORAGE)) {
            Toast.makeText(PantallaPrincipal.this, "El permniso es necesario para almacenar tus recursos en el téfono. Por favor, autoriza este permiso en Ajustes del Sistema.", Toast.LENGTH_LONG).show();
        } else {
            ActivityCompat.requestPermissions(PantallaPrincipal.this, new String[]{android.Manifest.permission.WRITE_EXTERNAL_STORAGE}, PERMISSION_REQUEST_CODE);
        }
    }

    @Override
    public void onRequestPermissionsResult(int requestCode, String permissions[], int[] grantResults) {
        switch (requestCode) {
            case PERMISSION_REQUEST_CODE:
                if (grantResults.length > 0 && grantResults[0] == PackageManager.PERMISSION_GRANTED) {
                    Log.d(tag, "Permiso concedido.");
                } else {
                    Log.d(tag, "Permiso denegado.");
                }
                break;
        }
    }

    private void mostrarSnack(View view, String msg){
        Snackbar.make(view, msg, Snackbar.LENGTH_LONG)
                .setAction("Action", null).show();
    }

    private void asignarIcono(ImageView imageView, Rocket rocket, int pos){
        ArrayList<Recurso> recursos;
        AdministradorBD db = new AdministradorBD(getApplicationContext());
        recursos = db.obtenerRecursos(rocket.obtenerID());

        if(pos == 0){
            if(recursos.size()>0){
                if(recursos.get(0).obtenerTipo().equals("IMG")){
                    imageView.setImageDrawable(getResources().getDrawable(R.mipmap.picture, PantallaPrincipal.this.getTheme()));
                }else if(recursos.get(0).obtenerTipo().equals("PDF")){
                    imageView.setImageDrawable(getResources().getDrawable(R.mipmap.pdf, PantallaPrincipal.this.getTheme()));
                }else if(recursos.get(0).obtenerTipo().equals("DOC")){
                    imageView.setImageDrawable(getResources().getDrawable(R.mipmap.doc_icon, PantallaPrincipal.this.getTheme()));
                }else if(recursos.get(0).obtenerTipo().equals("AUD")){
                    imageView.setImageDrawable(getResources().getDrawable(R.mipmap.audio_icon, PantallaPrincipal.this.getTheme()));
                }else if(recursos.get(0).obtenerTipo().equals("VID")){
                    imageView.setImageDrawable(getResources().getDrawable(R.mipmap.video_icon, PantallaPrincipal.this.getTheme()));
                }else if(recursos.get(0).obtenerTipo().equals("LNK")){
                    imageView.setImageDrawable(getResources().getDrawable(R.mipmap.link_icon, PantallaPrincipal.this.getTheme()));
                }else if(recursos.get(0).obtenerTipo().equals("TXT")){
                    imageView.setImageDrawable(getResources().getDrawable(R.mipmap.texto_icon, PantallaPrincipal.this.getTheme()));
                }else{
                    imageView.setImageDrawable(getResources().getDrawable(R.mipmap.mas, PantallaPrincipal.this.getTheme()));
                }
            }else{
                imageView.setImageDrawable(null);
            }
            return;
        }else if(pos == 1){
            if(recursos.size()>=2){
                if(recursos.get(1).obtenerTipo().equals("IMG")){
                    imageView.setImageDrawable(getResources().getDrawable(R.mipmap.picture, PantallaPrincipal.this.getTheme()));
                }else if(recursos.get(1).obtenerTipo().equals("PDF")){
                    imageView.setImageDrawable(getResources().getDrawable(R.mipmap.pdf, PantallaPrincipal.this.getTheme()));
                }else if(recursos.get(1).obtenerTipo().equals("DOC")){
                    imageView.setImageDrawable(getResources().getDrawable(R.mipmap.doc_icon, PantallaPrincipal.this.getTheme()));
                }else if(recursos.get(1).obtenerTipo().equals("AUD")){
                    imageView.setImageDrawable(getResources().getDrawable(R.mipmap.audio_icon, PantallaPrincipal.this.getTheme()));
                }else if(recursos.get(1).obtenerTipo().equals("VID")){
                    imageView.setImageDrawable(getResources().getDrawable(R.mipmap.video_icon, PantallaPrincipal.this.getTheme()));
                }else if(recursos.get(1).obtenerTipo().equals("LNK")){
                    imageView.setImageDrawable(getResources().getDrawable(R.mipmap.link_icon, PantallaPrincipal.this.getTheme()));
                }else if(recursos.get(1).obtenerTipo().equals("TXT")){
                    imageView.setImageDrawable(getResources().getDrawable(R.mipmap.texto_icon, PantallaPrincipal.this.getTheme()));
                }else{
                    imageView.setImageDrawable(getResources().getDrawable(R.mipmap.mas, PantallaPrincipal.this.getTheme()));
                }
            }else {
                imageView.setImageDrawable(null);
            }
            return;
        }else{
            if(recursos.size()>=3){
                if(recursos.get(2).obtenerTipo().equals("IMG")){
                    imageView.setImageDrawable(getResources().getDrawable(R.mipmap.picture, PantallaPrincipal.this.getTheme()));
                }else if(recursos.get(2).obtenerTipo().equals("PDF")){
                    imageView.setImageDrawable(getResources().getDrawable(R.mipmap.pdf, PantallaPrincipal.this.getTheme()));
                }else if(recursos.get(2).obtenerTipo().equals("DOC")){
                    imageView.setImageDrawable(getResources().getDrawable(R.mipmap.doc_icon, PantallaPrincipal.this.getTheme()));
                }else if(recursos.get(2).obtenerTipo().equals("AUD")){
                    imageView.setImageDrawable(getResources().getDrawable(R.mipmap.audio_icon, PantallaPrincipal.this.getTheme()));
                }else if(recursos.get(2).obtenerTipo().equals("VID")){
                    imageView.setImageDrawable(getResources().getDrawable(R.mipmap.video_icon, PantallaPrincipal.this.getTheme()));
                }else if(recursos.get(2).obtenerTipo().equals("LNK")){
                    imageView.setImageDrawable(getResources().getDrawable(R.mipmap.link_icon, PantallaPrincipal.this.getTheme()));
                }else if(recursos.get(2).obtenerTipo().equals("TXT")){
                    imageView.setImageDrawable(getResources().getDrawable(R.mipmap.texto_icon, PantallaPrincipal.this.getTheme()));
                }else{
                    imageView.setImageDrawable(getResources().getDrawable(R.mipmap.mas, PantallaPrincipal.this.getTheme()));
                }
            }else {
                imageView.setImageDrawable(null);
            }
            return;
        }
    }

    @OnLifecycleEvent(Lifecycle.Event.ON_RESUME)
    public void refrescarRockets(){
        Log.d(tag, "Resumiendo...");
        cargarRockets();
    }

    @Override
    protected void onRestart(){
        super.onRestart();
        Log.d(tag, "Reiniciando...");
        cargarRockets();
    }

}
