/*
Licencia:
        Attribution-NonCommercial-ShareAlike 4.0 International (CC BY-NC-SA 4.0)
*/

package org.no_ip.arias.rockets;

import android.app.Activity;
import android.app.AlertDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.database.Cursor;
import android.graphics.Color;
import android.net.Uri;
import android.os.Bundle;
import android.os.Environment;
import android.os.StrictMode;
import android.provider.OpenableColumns;
import android.support.design.widget.FloatingActionButton;
import android.support.design.widget.Snackbar;
import android.support.v7.app.AppCompatActivity;
import android.support.v7.widget.Toolbar;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ArrayAdapter;
import android.widget.EditText;
import android.widget.ImageButton;
import android.widget.ListView;
import android.widget.RadioButton;
import android.widget.TextView;

import java.io.File;
import java.io.FileOutputStream;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;

public class PantallaReceptora extends AppCompatActivity {
    private String tag = "Actividad Receptora ";
    private ListView lista;
    private List<Rocket> listaRockets;
    private Intent intent;
    private String accion, tipo;
    private Recurso recurso;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_pantalla_receptora);
        Toolbar toolbar = (Toolbar) findViewById(R.id.toolbar);
        toolbar.setTitle("Seleccione Rocket:");
        setSupportActionBar(toolbar);

        lista = (ListView)findViewById(R.id.listaReceptora);
        cargarRockets();


        FloatingActionButton fab = (FloatingActionButton) findViewById(R.id.fab);
        fab.setImageResource(R.drawable.plus_icon);
        fab.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                mostrarDialogoCrearRocket(view);
            }
        });

        intent = getIntent();
        accion = intent.getAction();
        tipo = intent.getType();
    }

    private void cargarRockets(){
        AdministradorBD administradorBD = new AdministradorBD(getApplicationContext());

        listaRockets = administradorBD.obtenerRockets();

        ArrayAdapter<Rocket> adaptador = new ArrayAdapter<Rocket>(this, R.layout.esquema_fila_receptora, R.id.textoLista, listaRockets){
            @Override
            public View getView(int position, View convertView, ViewGroup parent){
                View view = super.getView(position, convertView, parent);
                final Rocket rocket = getItem(position);

                TextView tv = (TextView)view.findViewById(R.id.textoLista);
                tv.setText(rocket.obtenerApodo());

                tv.setOnClickListener(new View.OnClickListener() {
                    @Override
                    public void onClick(View view) {
                    //Guardar el recurso en Rocket seleccionado
                    Log.d(tag, "Rocket: "+rocket.obtenerID());
                    if (Intent.ACTION_SEND.equals(accion) && tipo != null) {
                        if ("text/plain".equals(tipo)) {
                            gestionarTextoRecibido(intent, rocket);
                        } else if (tipo.startsWith("image/")) {
                            gestionarImagenRecibida(intent, rocket);
                        } else if (tipo.startsWith("application/") || tipo.startsWith("video/") || tipo.startsWith("audio/")){
                            gestionarDocumentoRecibido(intent, rocket);
                        }
                    } else if (Intent.ACTION_SEND_MULTIPLE.equals(accion) && tipo != null) {
                        if (tipo.startsWith("image/")) {
                            gestionarImagenesRecibidas(intent);
                        }
                    } else {
                        // gestionar otros intents, como los iniciados desde el home screen
                    }
                    }
                });

                if(rocket.obtenerColor() == 1)
                    view.setBackgroundColor(getResources().getColor(R.color.color1, PantallaReceptora.this.getTheme()));
                else if(rocket.obtenerColor() == 2)
                    view.setBackgroundColor(getResources().getColor(R.color.color2, PantallaReceptora.this.getTheme()));
                else if(rocket.obtenerColor() == 3)
                    view.setBackgroundColor(getResources().getColor(R.color.color3, PantallaReceptora.this.getTheme()));
                else
                    view.setBackgroundColor(getResources().getColor(R.color.color4, PantallaReceptora.this.getTheme()));

                return view;
            }
        };

        lista.setAdapter(adaptador);
    }

    void gestionarTextoRecibido(Intent intent, Rocket rocket){
        String textoCompartido = intent.getStringExtra(Intent.EXTRA_TEXT);
        if (textoCompartido != null) {
            Log.d(tag, textoCompartido);
            recurso = new Recurso();
            recurso.establecerRocket_ID(rocket.obtenerID());
            String inicio = textoCompartido.substring(0, 4);
            if(inicio.equals("http")){
                recurso.establecerTipo("LNK");
            }else {
                recurso.establecerTipo("TXT");
            }
            recurso.establecerTextoContenido(textoCompartido);
            AdministradorBD db = new AdministradorBD(getApplicationContext());
            long id = db.insertarRecursoDB(recurso, rocket.obtenerID());
        }
        this.finish();
    }

    void gestionarImagenRecibida(Intent intent, Rocket rocket){
        Uri uriImagen = (Uri) intent.getParcelableExtra(Intent.EXTRA_STREAM);
        if (uriImagen != null) {
            recurso = new Recurso();
            recurso.establecerNombreArchivo(obtenerNombreArchivo(uriImagen));
            recurso.establecerRocket_ID(rocket.obtenerID());
            recurso.establecerRuta(this.getExternalFilesDir(Environment.DIRECTORY_DOCUMENTS)+"/"+rocket.obtenerID()+"_"+rocket.obtenerNombre());
            recurso.establecerTipo("IMG");


            File archivoImagen = new File(this.getExternalFilesDir(Environment.DIRECTORY_DOCUMENTS)+"/"+rocket.obtenerID()+"_"+rocket.obtenerNombre(), obtenerNombreArchivo(uriImagen));
            try {
                InputStream entrada = getContentResolver().openInputStream(uriImagen);
                OutputStream salida = new FileOutputStream(archivoImagen);
                byte[] buf = new byte[1024];
                int len, tamanio = 0;
                while((len=entrada.read(buf))>0){
                    salida.write(buf,0,len);
                    tamanio+=1024;
                }
                recurso.establecerTamanio(tamanio);
                salida.close();
                entrada.close();

                AdministradorBD db = new AdministradorBD(getApplicationContext());
                long id = db.insertarRecursoDB(recurso, rocket.obtenerID());

            }catch (Exception ex){
                Log.d(tag, "Error manipulando archivo.");
            }
        }
        this.finish();
    }

    void gestionarDocumentoRecibido(Intent intent, Rocket rocket){

        Uri uriDoc = (Uri) intent.getParcelableExtra(Intent.EXTRA_STREAM);
        if (uriDoc != null) {
            recurso = new Recurso();
            recurso.establecerNombreArchivo(obtenerNombreArchivo(uriDoc));
            recurso.establecerRocket_ID(rocket.obtenerID());
            recurso.establecerRuta(this.getExternalFilesDir(Environment.DIRECTORY_DOCUMENTS)+"/"+rocket.obtenerID()+"_"+rocket.obtenerNombre());
            recurso.establecerTipo(obtenerTipo(obtenerNombreArchivo(uriDoc)));

            File archivoImagen = new File(this.getExternalFilesDir(Environment.DIRECTORY_DOCUMENTS)+"/"+rocket.obtenerID()+"_"+rocket.obtenerNombre(), obtenerNombreArchivo(uriDoc));
            try {
                InputStream entrada = getContentResolver().openInputStream(uriDoc);
                OutputStream salida = new FileOutputStream(archivoImagen);
                byte[] buf = new byte[1024];
                int len, tamanio = 0;
                while((len=entrada.read(buf))>0){
                    salida.write(buf,0,len);
                    tamanio+=1024;
                }
                recurso.establecerTamanio(tamanio);
                salida.close();
                entrada.close();

                AdministradorBD db = new AdministradorBD(getApplicationContext());
                long id = db.insertarRecursoDB(recurso, rocket.obtenerID());

            }catch (Exception ex){
                Log.d(tag, "Error manipulando archivo.");
            }
        }else{
            Log.d(tag,"Uri null");
        }
        this.finish();
    }

    void gestionarImagenesRecibidas(Intent intent){
        ArrayList<Uri> uriImagenes = intent.getParcelableArrayListExtra(Intent.EXTRA_STREAM);
        if (uriImagenes != null) {
            // Update UI to reflect multiple images being shared
        }
    }

    private String obtenerNombreArchivo(Uri uri) {
        String result = null;
        if (uri.getScheme().equals("content")) {
            Cursor cursor = getContentResolver().query(uri, null, null, null, null);
            try {
                if (cursor != null && cursor.moveToFirst()) {
                    result = cursor.getString(cursor.getColumnIndex(OpenableColumns.DISPLAY_NAME));
                }
            } finally {
                cursor.close();
            }
        }
        if (result == null) {
            result = uri.getPath();
            int cut = result.lastIndexOf('/');
            if (cut != -1) {
                result = result.substring(cut + 1);
            }
        }
        return result;
    }

    private void mostrarDialogoCrearRocket(final View view) {
        AlertDialog.Builder builder = new AlertDialog.Builder(PantallaReceptora.this);
        builder.setTitle("Crear Rocket");

        final View customLayout = getLayoutInflater().inflate(R.layout.dialogo_nuevo_rocket, null);
        builder.setView(customLayout);

        builder.setPositiveButton("Crear", new DialogInterface.OnClickListener() {
            @Override
            public void onClick(DialogInterface dialogInterface, int i) {
                EditText nombre_et = (EditText) customLayout.findViewById(R.id.nombre_dialogo);
                RadioButton radioBtn1 = (RadioButton) customLayout.findViewById(R.id.radioBtn1);
                RadioButton radioBtn2 = (RadioButton) customLayout.findViewById(R.id.radioBtn2);
                RadioButton radioBtn3 = (RadioButton) customLayout.findViewById(R.id.radioBtn3);
                RadioButton radioBtn4 = (RadioButton) customLayout.findViewById(R.id.radioBtn4);

                int color=1;
                if(radioBtn1.isChecked())
                    color = 1;
                else if(radioBtn2.isChecked())
                    color = 2;
                else if(radioBtn3.isChecked())
                    color = 3;
                else if(radioBtn4.isChecked())
                    color = 4;

                Rocket rocket = new Rocket(nombre_et.getText().toString(), color);

                AdministradorBD db = new AdministradorBD(getApplicationContext());
                long id = db.insertarRocketDB(rocket);
                Log.d(tag, "ID:" + id);

                rocket = db.obtenerRocketDB(id);
                Log.d(tag, rocket.obtenerNombre() + " " + rocket.obtenerColor());

                crearDirectorio(rocket);

                cargarRockets();

                mostrarSnack(view, "Rocket creado.");
            }
        });

        builder.setNegativeButton("Cancelar", new DialogInterface.OnClickListener() {
            @Override
            public void onClick(DialogInterface dialogInterface, int i) {
                //AdministradorBD db = new AdministradorBD(getApplicationContext());
                //Log.d(tag, "Total Rockets: "+db.contarRockets());
                mostrarSnack(view, "Rocket no creado.");
            }
        });

        builder.create();

        builder.show();
    }

    private void crearDirectorio(Rocket rocket){
        File file = new File(this.getExternalFilesDir(Environment.DIRECTORY_DOCUMENTS), rocket.obtenerID()+"_"+rocket.obtenerNombre());
        if (!file.mkdir()) {
            Log.d(tag, "Operacion fallida.");
        }else{
            Log.d(tag, "Operacion Correcta.");
            Log.d(tag, "Ruta:"+file.getPath());
        }
    }

    private void mostrarSnack(View view, String msg){
        Snackbar.make(view, msg, Snackbar.LENGTH_LONG)
                .setAction("Action", null).show();
    }

    private String obtenerTipo(String nombreArchivo){
        String extensionArchivo = nombreArchivo.substring(nombreArchivo.length()-3).toLowerCase();
        if(extensionArchivo.equals("pdf"))
            return "PDF";
        else if(extensionArchivo.equals("jpg") || extensionArchivo.equals("png") || extensionArchivo.equals("gif") || extensionArchivo.equals("bmp")) {
            return "IMG";
        }else if(extensionArchivo.equals("mp4") || extensionArchivo.equals("avi") || extensionArchivo.equals("mpg") || extensionArchivo.equals("mkv")){
                return "VID";
        }else if(extensionArchivo.equals("mp3") || extensionArchivo.equals("ogg") || extensionArchivo.equals("wav") || extensionArchivo.equals("acc")){
            return "AUD";
        }else if(extensionArchivo.equals("doc") || extensionArchivo.equals("xls") || extensionArchivo.equals("odt") || extensionArchivo.equals("ods") || extensionArchivo.equals("ppt")){
            return "DOC";
        }else{
            return "OTRO";
        }
    }
}
